import React, { Component } from 'react';
import PropTypes from 'prop-types';
import Helmet from 'react-helmet';
/**
 * Wrapper component containing HTML metadata and boilerplate tags.
 * Used in server-side code only to wrap the string output of the
 * rendered route component.
 *
 * The only thing this component doesn't (and can't) include is the
 * HTML doctype declaration, which is added to the rendered output
 * by the server.js file.
 */
export default class Html extends Component {
  static propTypes = {
    assets: PropTypes.object,
    component: PropTypes.node,
    baseDomain: PropTypes.string,
  };

  render() {
    const { assets } = this.props;
    const head = Helmet.rewind();

    return (
      <html lang="en-us">
        <head>
          <link rel="icon" type="image/png" href="/rstatic/favicon.png" />

          {Object.keys(assets.styles).map((style, key) => (
            <link
              href={assets.styles[style]}
              key={key}
              media="screen, projection"
              rel="stylesheet"
              type="text/css"
              charSet="UTF-8"
            />
          ))}

          <script
            dangerouslySetInnerHTML={{
              __html: `window.__env={
                apiHost: '${process.env.API_HOST}',
                apiPort: '${process.env.API_PORT}',
                dataApiUrl: '${process.env.DATA_API_URL}',
                accessKey: '${process.env.ACCESS_KEY}',
                consoleMode: '${process.env.CONSOLE_MODE}',
                nodeEnv: '${process.env.NODE_ENV}',
                devDataApiUrl: '${process.env.DEV_DATA_API_URL}',
                urlPrefix: '${process.env.URL_PREFIX}'
              };`,
            }}
          />
        </head>
        <body>
          <style
            dangerouslySetInnerHTML={{
              __html: `
            .content {
              display: 'none';
              opacity: 0;
              transition: opacity .20s linear;
            }
            .content.show {
              display: 'block';
              opacity: 1;
              transition: opacity .20s linear;
            }
          `,
            }}
          />
          <div
            id="loading"
            dangerouslySetInnerHTML={{
              __html: `<div class="page-loading" style="
                min-height: 100vh;
                width: 100%;
                display: flex;
                align-items: center;
                font-family: sans-serif;
                justify-content: center;
            ">
            <span class="" style="
                font-size: 2em;
                margin-top: -3em;
                color: #848484;
            ">
              Loading...
              </span>
            </div>`,
            }}
          />

          <div id="content" className="content" />
          <script src={assets.javascript.main} charSet="UTF-8" />
        </body>
      </html>
    );
  }
}
